@@ -1,18 +1,22 @@
 open! Import
 
-module type Key = sig
-  type t [@@deriving_inline compare, sexp_of]
-  include
-  sig
-    [@@@ocaml.warning "-32"]
-    val compare : t -> t -> int
-    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-  end[@@ocaml.doc "@inline"]
-  [@@@end]
+module Key = struct
+  module type S = sig
+    type t [@@deriving_inline compare, sexp_of]
+    include
+      sig
+        [@@@ocaml.warning "-32"]
+        val compare : t -> t -> int
+        val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+      end[@@ocaml.doc "@inline"]
+    [@@@end]
 
-  (** Two [t]s that [compare] equal must have equal hashes for the hashtable
-      to behave properly. *)
-  val hash : t -> int
+    (** Two [t]s that [compare] equal must have equal hashes for the hashtable
+        to behave properly. *)
+    val hash : t -> int
+  end
+
+  type 'a t = (module S with type t = 'a)
 end
 
 module type Accessors = sig
@@ -29,8 +33,8 @@ module type Accessors = sig
       [iter], [iter_keys], [iteri]) will raise an exception. *)
   val fold : ('a, 'b) t -> init:'c -> f:(key:'a key -> data:'b -> 'c -> 'c) -> 'c
 
-  val iter_keys : ('a,  _) t -> f:(    'a key            -> unit) -> unit
-  val iter      : ( _, 'b) t -> f:(                   'b -> unit) -> unit
+  val iter_keys : ('a, _) t -> f:('a key -> unit) -> unit
+  val iter : (_, 'b) t -> f:('b -> unit) -> unit
 
   (** Iterates over both keys and values.
 
@@ -44,26 +48,28 @@ module type Accessors = sig
       5-6
       - : unit = ()
       v} *)
-  val iteri     : ('a, 'b) t -> f:(key:'a key -> data:'b -> unit) -> unit
+  val iteri : ('a, 'b) t -> f:(key:'a key -> data:'b -> unit) -> unit
 
-  val existsi  : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> bool
-  val exists   : (_ , 'b) t -> f:(                   'b -> bool) -> bool
+  val existsi : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> bool
+  val exists : (_, 'b) t -> f:('b -> bool) -> bool
   val for_alli : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> bool
-  val for_all  : (_ , 'b) t -> f:(                   'b -> bool) -> bool
-  val counti   : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> int
-  val count    : (_ , 'b) t -> f:(                   'b -> bool) -> int
-
+  val for_all : (_, 'b) t -> f:('b -> bool) -> bool
+  val counti : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> int
+  val count : (_, 'b) t -> f:('b -> bool) -> int
   val length : (_, _) t -> int
   val is_empty : (_, _) t -> bool
   val mem : ('a, _) t -> 'a key -> bool
   val remove : ('a, _) t -> 'a key -> unit
+  val choose : ('a, 'b) t -> ('a key * 'b) option
+  val choose_exn : ('a, 'b) t -> 'a key * 'b
 
   (** Sets the given [key] to [data]. *)
-  val set          : ('a, 'b) t -> key:'a key -> data:'b -> unit
+  val set : ('a, 'b) t -> key:'a key -> data:'b -> unit
 
   (** [add] and [add_exn] leave the table unchanged if the key was already present. *)
-  val add          : ('a, 'b) t -> key:'a key -> data:'b -> [ `Ok | `Duplicate ]
-  val add_exn      : ('a, 'b) t -> key:'a key -> data:'b -> unit
+  val add : ('a, 'b) t -> key:'a key -> data:'b -> [ `Ok | `Duplicate ]
+
+  val add_exn : ('a, 'b) t -> key:'a key -> data:'b -> unit
 
   (** [change t key ~f] changes [t]'s value for [key] to be [f (find t key)]. *)
   val change : ('a, 'b) t -> 'a key -> f:('b option -> 'b option) -> unit
@@ -102,24 +108,23 @@ module type Accessors = sig
   val filter_map : ('a, 'b) t -> f:('b -> 'c option) -> ('a, 'c) t
 
   (** Like [filter_map], but the function [f] takes both key and data as arguments. *)
-  val filter_mapi
-    : ('a, 'b) t -> f:(key:'a key -> data:'b -> 'c option) -> ('a, 'c) t
+  val filter_mapi : ('a, 'b) t -> f:(key:'a key -> data:'b -> 'c option) -> ('a, 'c) t
 
   val filter_keys : ('a, 'b) t -> f:('a key -> bool) -> ('a, 'b) t
-  val filter      : ('a, 'b) t -> f:('b -> bool) -> ('a, 'b) t
-  val filteri     : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> ('a, 'b) t
+  val filter : ('a, 'b) t -> f:('b -> bool) -> ('a, 'b) t
+  val filteri : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> ('a, 'b) t
 
   (** Returns new tables with bound values partitioned by [f] applied to the bound
       values. *)
   val partition_map
     :  ('a, 'b) t
-    -> f:('b -> [`Fst of 'c | `Snd of 'd])
+    -> f:('b -> [ `Fst of 'c | `Snd of 'd ])
     -> ('a, 'c) t * ('a, 'd) t
 
   (** Like [partition_map], but the function [f] takes both key and data as arguments. *)
   val partition_mapi
     :  ('a, 'b) t
-    -> f:(key:'a key -> data:'b -> [`Fst of 'c | `Snd of 'd])
+    -> f:(key:'a key -> data:'b -> [ `Fst of 'c | `Snd of 'd ])
     -> ('a, 'c) t * ('a, 'd) t
 
   (** Returns a pair of tables [(t1, t2)], where [t1] contains all the elements of the
@@ -128,7 +133,9 @@ module type Accessors = sig
 
   (** Like [partition_tf], but the function [f] takes both key and data as arguments. *)
   val partitioni_tf
-    : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> ('a, 'b) t * ('a, 'b) t
+    :  ('a, 'b) t
+    -> f:(key:'a key -> data:'b -> bool)
+    -> ('a, 'b) t * ('a, 'b) t
 
   (** [find_or_add t k ~default] returns the data associated with key [k] if it is in the
       table [t], and otherwise assigns [k] the value returned by [default ()]. *)
@@ -176,13 +183,13 @@ module type Accessors = sig
       sets of keys of [h1] and [h2] for which [d(k)] is not None, where:
 
       d(k) =
-      - [f ~key:k (Some d1) None]
+      - [f ~key:k (`Left d1)]
         if [k] in [h1] maps to d1, and [h2] does not have data for [k];
 
-      - [f ~key:k None (Some d2)]
+      - [f ~key:k (`Right d2)]
         if [k] in [h2] maps to d2, and [h1] does not have data for [k];
 
-      - [f ~key:k (Some d1) (Some d2)]
+      - [f ~key:k (`Both (d1, d2))]
         otherwise, where [k] in [h1] maps to [d1] and [k] in [h2] maps to [d2].
 
       Each key [k] is mapped to a single piece of data [x], where [d(k) = Some x].
@@ -208,7 +215,10 @@ module type Accessors = sig
 
   (** Every [key] in [src] will be removed or set in [dst] according to the return value
       of [f]. *)
-  type 'a merge_into_action = Remove | Set_to of 'a
+  type 'a merge_into_action =
+    | Remove
+    | Set_to of 'a
+
 
   val merge_into
     :  src:('k, 'a) t
@@ -223,23 +233,27 @@ module type Accessors = sig
   val data : (_, 'b) t -> 'b list
 
   (** [filter_inplace t ~f] removes all the elements from [t] that don't satisfy [f]. *)
-  val filter_keys_inplace : ('a,  _) t -> f:('a key -> bool) -> unit
-  val filter_inplace      : ( _, 'b) t -> f:('b -> bool) -> unit
-  val filteri_inplace     : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> unit
+  val filter_keys_inplace : ('a, _) t -> f:('a key -> bool) -> unit
+
+  val filter_inplace : (_, 'b) t -> f:('b -> bool) -> unit
+  val filteri_inplace : ('a, 'b) t -> f:(key:'a key -> data:'b -> bool) -> unit
 
   (** [map_inplace t ~f] applies [f] to all elements in [t], transforming them in
       place. *)
-  val map_inplace  : (_,  'b) t -> f:(                   'b -> 'b) -> unit
+  val map_inplace : (_, 'b) t -> f:('b -> 'b) -> unit
+
   val mapi_inplace : ('a, 'b) t -> f:(key:'a key -> data:'b -> 'b) -> unit
 
   (** [filter_map_inplace] combines the effects of [map_inplace] and [filter_inplace]. *)
-  val filter_map_inplace  : (_,  'b) t -> f:(                   'b -> 'b option) -> unit
+  val filter_map_inplace : (_, 'b) t -> f:('b -> 'b option) -> unit
+
   val filter_mapi_inplace : ('a, 'b) t -> f:(key:'a key -> data:'b -> 'b option) -> unit
 
   (** [equal t1 t2 f] and [similar t1 t2 f] both return true iff [t1] and [t2] have the
       same keys and for all keys [k], [f (find_exn t1 k) (find_exn t2 k)].  [equal] and
       [similar] only differ in their types. *)
-  val equal   : ('a, 'b ) t -> ('a, 'b ) t -> ('b  -> 'b  -> bool) -> bool
+  val equal : ('a, 'b) t -> ('a, 'b) t -> ('b -> 'b -> bool) -> bool
+
   val similar : ('a, 'b1) t -> ('a, 'b2) t -> ('b1 -> 'b2 -> bool) -> bool
 
   (** Returns the list of all (key, data) pairs for given hashtable. *)
@@ -250,8 +264,10 @@ module type Accessors = sig
     -> 'b Validate.check
     -> ('a, 'b) t Validate.check
 
+
   (** [remove_if_zero]'s default is [false]. *)
   val incr : ?by:int -> ?remove_if_zero:bool -> ('a, int) t -> 'a key -> unit
+
   val decr : ?by:int -> ?remove_if_zero:bool -> ('a, int) t -> 'a key -> unit
 end
 
@@ -275,13 +291,13 @@ end
 
 type ('key, 'data, 'z) create_options =
   ?growth_allowed:bool (** defaults to [true] *)
-  -> ?size:int (** initial size -- default 128 *)
-  -> (module Key with type t = 'key)
+  -> ?size:int (** initial size -- default 0 *)
+  -> 'key Key.t
   -> 'z
 
 type ('key, 'data, 'z) create_options_without_first_class_module =
   ?growth_allowed:bool (** defaults to [true] *)
-  -> ?size:int (** initial size -- default 128 *)
+  -> ?size:int (** initial size -- default 0 *)
   -> 'z
 
 module type Creators_generic = sig
@@ -291,21 +307,18 @@ module type Creators_generic = sig
 
   val create : ('a key, 'b, unit -> ('a, 'b) t) create_options
 
+
   val of_alist
-    :  ('a key,
-        'b,
-        ('a key * 'b) list
-        -> [ `Ok of ('a, 'b) t
-           | `Duplicate_key of 'a key
-           ]) create_options
+    : ( 'a key
+      , 'b
+      , ('a key * 'b) list -> [ `Ok of ('a, 'b) t | `Duplicate_key of 'a key ] )
+        create_options
 
   val of_alist_report_all_dups
-    : ('a key,
-       'b,
-       ('a key * 'b) list
-       -> [ `Ok of ('a, 'b) t
-          | `Duplicate_keys of 'a key list
-          ]) create_options
+    : ( 'a key
+      , 'b
+      , ('a key * 'b) list -> [ `Ok of ('a, 'b) t | `Duplicate_keys of 'a key list ] )
+        create_options
 
   val of_alist_or_error
     : ('a key, 'b, ('a key * 'b) list -> ('a, 'b) t Or_error.t) create_options
@@ -319,46 +332,44 @@ module type Creators_generic = sig
   (** {[ create_mapped get_key get_data [x1,...,xn]
          = of_alist [get_key x1, get_data x1; ...; get_key xn, get_data xn] ]} *)
   val create_mapped
-    : ('a key,
-       'b,
-       get_key:('r -> 'a key)
-       -> get_data:('r -> 'b)
-       -> 'r list
-       -> [ `Ok of ('a, 'b) t
-          | `Duplicate_keys of 'a key list ]) create_options
+    : ( 'a key
+      , 'b
+      , get_key:('r -> 'a key)
+        -> get_data:('r -> 'b)
+        -> 'r list
+        -> [ `Ok of ('a, 'b) t | `Duplicate_keys of 'a key list ] )
+        create_options
+
 
   (** {[ create_with_key ~get_key [x1,...,xn]
          = of_alist [get_key x1, x1; ...; get_key xn, xn] ]} *)
   val create_with_key
-    : ('a key,
-       'r,
-       get_key:('r -> 'a key)
-       -> 'r list
-       -> [ `Ok of ('a, 'r) t
-          | `Duplicate_keys of 'a key list ]) create_options
+    : ( 'a key
+      , 'r
+      , get_key:('r -> 'a key)
+        -> 'r list
+        -> [ `Ok of ('a, 'r) t | `Duplicate_keys of 'a key list ] )
+        create_options
 
   val create_with_key_or_error
-    : ('a key,
-       'r,
-       get_key:('r -> 'a key)
-       -> 'r list
-       -> ('a, 'r) t Or_error.t) create_options
+    : ( 'a key
+      , 'r
+      , get_key:('r -> 'a key) -> 'r list -> ('a, 'r) t Or_error.t )
+        create_options
 
   val create_with_key_exn
-    : ('a key,
-       'r,
-       get_key:('r -> 'a key)
-       -> 'r list
-       -> ('a, 'r) t) create_options
+    : ('a key, 'r, get_key:('r -> 'a key) -> 'r list -> ('a, 'r) t) create_options
+
 
   val group
-    : ('a key,
-       'b,
-       get_key:('r -> 'a key)
-       -> get_data:('r -> 'b)
-       -> combine:('b -> 'b -> 'b)
-       -> 'r list
-       -> ('a, 'b) t) create_options
+    : ( 'a key
+      , 'b
+      , get_key:('r -> 'a key)
+        -> get_data:('r -> 'b)
+        -> combine:('b -> 'b -> 'b)
+        -> 'r list
+        -> ('a, 'b) t )
+        create_options
 end
 
 module type Creators = sig
@@ -377,8 +388,8 @@ module type Creators = sig
       v} *)
   val create
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> ('a, 'b) t
 
   (** Example:
@@ -389,12 +400,10 @@ module type Creators = sig
       v} *)
   val of_alist
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> ('a * 'b) list
-    -> [ `Ok of ('a, 'b) t
-       | `Duplicate_key of 'a
-       ]
+    -> [ `Ok of ('a, 'b) t | `Duplicate_key of 'a ]
 
   (** Whereas [of_alist] will report [Duplicate_key] no matter how many dups there are in
       your list, [of_alist_report_all_dups] will report each and every duplicate entry.
@@ -410,24 +419,22 @@ module type Creators = sig
       v} *)
   val of_alist_report_all_dups
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> ('a * 'b) list
-    -> [ `Ok of ('a, 'b) t
-       | `Duplicate_keys of 'a list
-       ]
+    -> [ `Ok of ('a, 'b) t | `Duplicate_keys of 'a list ]
 
   val of_alist_or_error
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> ('a * 'b) list
     -> ('a, 'b) t Or_error.t
 
   val of_alist_exn
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> ('a * 'b) list
     -> ('a, 'b) t
 
@@ -445,8 +452,8 @@ module type Creators = sig
       v} *)
   val of_alist_multi
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> ('a * 'b) list
     -> ('a, 'b list) t
 
@@ -477,37 +484,35 @@ module type Creators = sig
       v} *)
   val create_mapped
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> get_key:('r -> 'a)
     -> get_data:('r -> 'b)
     -> 'r list
-    -> [ `Ok of ('a, 'b) t
-       | `Duplicate_keys of 'a list ]
+    -> [ `Ok of ('a, 'b) t | `Duplicate_keys of 'a list ]
 
   (** {[ create_with_key ~get_key [x1;...;xn]
          = of_alist [get_key x1, x1; ...; get_key xn, xn] ]} *)
   val create_with_key
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> get_key:('r -> 'a)
     -> 'r list
-    -> [ `Ok of ('a, 'r) t
-       | `Duplicate_keys of 'a list ]
+    -> [ `Ok of ('a, 'r) t | `Duplicate_keys of 'a list ]
 
   val create_with_key_or_error
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> get_key:('r -> 'a)
     -> 'r list
     -> ('a, 'r) t Or_error.t
 
   val create_with_key_exn
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> get_key:('r -> 'a)
     -> 'r list
     -> ('a, 'r) t
@@ -531,8 +536,8 @@ module type Creators = sig
        v} *)
   val group
     :  ?growth_allowed:bool (** defaults to [true] *)
-    -> ?size:int (** initial size -- default 128 *)
-    -> (module Key with type t = 'a)
+    -> ?size:int (** initial size -- default 0 *)
+    -> 'a Key.t
     -> get_key:('r -> 'a)
     -> get_data:('r -> 'b)
     -> combine:('b -> 'b -> 'b)
@@ -541,11 +546,9 @@ module type Creators = sig
 end
 
 module type S_without_submodules = sig
-
   val hash : 'a -> int
   val hash_param : int -> int -> 'a -> int
 
-
   type ('a, 'b) t
 
   (** We provide a [sexp_of_t] but not a [t_of_sexp] for this type because one needs to be
@@ -554,75 +557,73 @@ module type S_without_submodules = sig
       polymorphic comparison and and polymorphic hashing. *)
   val sexp_of_t : ('a -> Sexp.t) -> ('b -> Sexp.t) -> ('a, 'b) t -> Sexp.t
 
-  include Creators
-    with type ('a, 'b) t  := ('a, 'b) t
+  include Creators with type ('a, 'b) t := ('a, 'b) t (** @inline *)
+
+  include
+    Accessors with type ('a, 'b) t := ('a, 'b) t with type 'a key = 'a
   (** @inline *)
 
-  include Accessors
-    with type ('a, 'b) t := ('a, 'b) t
-    with type 'a key = 'a
+
+  include
+    Multi with type ('a, 'b) t := ('a, 'b) t with type 'a key := 'a key
   (** @inline *)
 
-  include Multi
-    with type ('a, 'b) t := ('a, 'b) t
-    with type 'a key := 'a key
-  (** @inline *)
-
-  val hashable_s : ('key, _) t -> (module Key with type t = 'key)
+  val hashable_s : ('key, _) t -> 'key Key.t
 
   include Invariant.S2 with type ('a, 'b) t := ('a, 'b) t
-
 end
 
 module type S_poly = sig
-
   type ('a, 'b) t [@@deriving_inline sexp]
   include
-  sig
-    [@@@ocaml.warning "-32"]
-    include Ppx_sexp_conv_lib.Sexpable.S2 with type ('a,'b) t :=  ('a, 'b) t
-  end[@@ocaml.doc "@inline"]
+    sig
+      [@@@ocaml.warning "-32"]
+      include Ppx_sexp_conv_lib.Sexpable.S2 with type ('a,'b) t :=  ('a, 'b) t
+    end[@@ocaml.doc "@inline"]
   [@@@end]
 
   val hashable : 'a Hashable.t
 
   include Invariant.S2 with type ('a, 'b) t := ('a, 'b) t
 
-  include Creators_generic
-    with type ('a, 'b) t  := ('a, 'b) t
+  include
+    Creators_generic
+    with type ('a, 'b) t := ('a, 'b) t
     with type 'a key = 'a
-    with type ('key, 'data, 'z) create_options
-    := ('key, 'data, 'z) create_options_without_first_class_module
+    with type ('key, 'data, 'z) create_options :=
+      ('key, 'data, 'z) create_options_without_first_class_module
 
-  include Accessors
-    with type ('a, 'b) t := ('a, 'b) t
-    with type 'a key := 'a key
-
-  include Multi
-    with type ('a, 'b) t := ('a, 'b) t
-    with type 'a key := 'a key
+  include Accessors with type ('a, 'b) t := ('a, 'b) t with type 'a key := 'a key
+  include Multi with type ('a, 'b) t := ('a, 'b) t with type 'a key := 'a key
 end
 
 module type For_deriving = sig
   type ('k, 'v) t
 
-  module type Sexp_of_m = sig type t [@@deriving_inline sexp_of]
+  module type Sexp_of_m = sig
+    type t [@@deriving_inline sexp_of]
     include
-    sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-    end[@@ocaml.doc "@inline"]
-    [@@@end] end
+      sig [@@@ocaml.warning "-32"] val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+      end[@@ocaml.doc "@inline"]
+    [@@@end]
+  end
+
   module type M_of_sexp = sig
     type t [@@deriving_inline of_sexp]
     include
-    sig [@@@ocaml.warning "-32"] val t_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t
-    end[@@ocaml.doc "@inline"]
-    [@@@end] include Key with type t := t
+      sig [@@@ocaml.warning "-32"] val t_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t
+      end[@@ocaml.doc "@inline"]
+    [@@@end]
+
+    include Key.S with type t := t
   end
+
   val sexp_of_m__t
     :  (module Sexp_of_m with type t = 'k)
     -> ('v -> Sexp.t)
     -> ('k, 'v) t
     -> Sexp.t
+
   val m__t_of_sexp
     :  (module M_of_sexp with type t = 'k)
     -> (Sexp.t -> 'v)
@@ -631,7 +632,6 @@ module type For_deriving = sig
 end
 
 module type Hashtbl = sig
-
   (** A hash table is a mutable data structure implementing a map between keys and values.
       It supports constant-time lookup and in-place modification.
 
@@ -708,22 +708,29 @@ module type Hashtbl = sig
 
   include S_without_submodules (** @inline *)
 
-  module type Accessors            = Accessors
-  module type Creators             = Creators
-  module type Key                  = Key
-  module type Multi                = Multi
-  module type S_poly               = S_poly
+  module type Accessors = Accessors
+  module type Creators = Creators
+  module type Key = Key.S [@@deprecated "[since 2019-03] Use [Hashtbl.Key.S]"]
+  module type Multi = Multi
+  module type S_poly = S_poly
   module type S_without_submodules = S_without_submodules
-
   module type For_deriving = For_deriving
 
-  type nonrec ('key, 'data, 'z) create_options =
-    ('key, 'data, 'z) create_options
+  module Key = Key
 
-  module Creators (Key : sig type 'a t val hashable : 'a t Hashable.t end) : sig
+  type nonrec ('key, 'data, 'z) create_options = ('key, 'data, 'z) create_options
+
+  module Creators (Key : sig
+      type 'a t
+
+      val hashable : 'a t Hashable.t
+    end) : sig
     type ('a, 'b) t_ = ('a Key.t, 'b) t
+
     val t_of_sexp : (Sexp.t -> 'a Key.t) -> (Sexp.t -> 'b) -> Sexp.t -> ('a, 'b) t_
-    include Creators_generic
+
+    include
+      Creators_generic
       with type ('a, 'b) t := ('a, 'b) t_
       with type 'a key := 'a Key.t
       with type ('key, 'data, 'a) create_options :=
@@ -754,11 +761,11 @@ module type Hashtbl = sig
   include For_deriving with type ('a, 'b) t := ('a, 'b) t
 
   (**/**)
+
   (*_ See the Jane Street Style Guide for an explanation of [Private] submodules:
 
     https://opensource.janestreet.com/standards/#private-submodules *)
   module Private : sig
-
     module type Creators_generic = Creators_generic
 
     type nonrec ('key, 'data, 'z) create_options_without_first_class_module =
