@@ -3,19 +3,25 @@
 
 open! Import
 
-type t = Neg | Zero | Pos | Nan [@@deriving_inline enumerate, hash]
+type t =
+  | Neg
+  | Zero
+  | Pos
+  | Nan
+[@@deriving_inline enumerate, hash]
 include
-sig
-  [@@@ocaml.warning "-32"]
-  val all : t list
-  val hash_fold_t :
-    Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-  val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-end[@@ocaml.doc "@inline"]
+  sig
+    [@@@ocaml.warning "-32"]
+    val all : t list
+    val hash_fold_t :
+      Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
+    val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
+  end[@@ocaml.doc "@inline"]
 [@@@end]
 
 (** This provides [to_string]/[of_string], sexp conversion, Map, Hashtbl, etc. *)
-include Identifiable.S with type t := t
+include
+  Identifiable.S with type t := t
 
 val of_int : int -> t
 
