@@ -7,22 +7,21 @@ open! Import
 
     [sexp_of_t] uses the form ["FILE:LINE:COL"], and does not have a corresponding
     [of_sexp]. *)
-type t
-  = Caml.Lexing.position
-  = { pos_fname : string
-    ; pos_lnum  : int
-    ; pos_bol   : int
-    ; pos_cnum  : int
-    }
+type t = Caml.Lexing.position =
+  { pos_fname : string
+  ; pos_lnum : int
+  ; pos_bol : int
+  ; pos_cnum : int
+  }
 [@@deriving_inline hash, sexp_of]
 include
-sig
-  [@@@ocaml.warning "-32"]
-  val hash_fold_t :
-    Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-  val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-  val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-end[@@ocaml.doc "@inline"]
+  sig
+    [@@@ocaml.warning "-32"]
+    val hash_fold_t :
+      Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
+    val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
+    val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
+  end[@@ocaml.doc "@inline"]
 [@@@end]
 
 include Comparable.S with type t := t
